#include "ca.h"

#include <math.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qlistview.h>

#include "doublespinbox.h"
#include "images.h"

CameraAssistant::CameraAssistant(QWidget* parent, const char* name, WFlags fl) : QMainWindow(parent, name, fl) {
	far=0,distance=5000,near=0,depth=0,hyperfocal=0,focallength=50,aperture=8,circle_of_confusion=0.008;
	units=UNITS_M;
	DoFMode=DOFMODE_DEPTH;
	FoVMode=FOVMODE_FOV;
	setCaption(tr("Camera Assistant"));
	
	config=new ConfigParser();
	config->configOpen();
	circle_of_confusion=config->circle_of_confusion;
	filmwidth=config->filmwidth;
	filmheight=config->filmheight;
	aspect=config->aspect_ratio;
	adjustments=config->adjustments;

	clickedValueMapper=new QSignalMapper(this);
	changedValueMapper=new QSignalMapper(this);
	connect(clickedValueMapper,SIGNAL(mapped(int)),this,SLOT(clickedValue(int)));
	connect(changedValueMapper,SIGNAL(mapped(int)),this,SLOT(changedValue(int)));
	
	myTabWidget = new QTabWidget(this,"My Tab Widget");
	setupTabDoF();
	setupTabFoV();
	setupTabPrefs();
	setupTabFilters();
	setupTabAbout();
	
	setCentralWidget(myTabWidget);

//	((QPEApplication*)parent)->setStylusOperation(distanceDoFSpinBox,QPEApplication::RightOnHold);

	cameracategoryPrefsComboBox->insertStringList(config->Categories);
	cameratypePrefsComboBox->insertStringList(config->CameraTypes);
	aspectPrefsComboBox->insertStringList(config->AspectRatios);
	cameracategoryPrefsComboBox->setCurrentItem(config->indexCameraCategory);
	cameratypePrefsComboBox->setCurrentItem(config->indexCameraType);
	aspectPrefsComboBox->setCurrentItem(config->indexAspectRatio);
	
	changedDoFMode(DOFMODE_DEPTH);
	changedFoVMode(FOVMODE_FOV);
	connect(myTabWidget,SIGNAL(currentChanged(QWidget *)),this,SLOT(changedPage()));
}

CameraAssistant::~CameraAssistant() {
}

// Depth of Field
void CameraAssistant::setupTabDoF() {
	QWidget *tabpage = new QWidget(this);
	QVBoxLayout *layout = new QVBoxLayout(tabpage);
	layout->setMargin(5);

	QHBoxLayout *hbox1 = new QHBoxLayout(-1,"hbox1");
	QVBoxLayout *vbox0 = new QVBoxLayout(-1,"vbox0");
	QVBoxLayout *vbox1 = new QVBoxLayout(-1,"vbox1");
	QVBoxLayout *vbox2 = new QVBoxLayout(-1,"vbox2");
	QVBoxLayout *vbox3 = new QVBoxLayout(-1,"vbox3");
	QHBoxLayout *hbox2 = new QHBoxLayout(-1,"hbox2");
	QVBoxLayout *vbox21 = new QVBoxLayout(-1,"vbox21");
	QVBoxLayout *vbox22 = new QVBoxLayout(-1,"vbox22");
	QVBoxLayout *vbox23 = new QVBoxLayout(-1,"vbox23");

	QLabel *myLabel = new QLabel("Label",tabpage);
	myLabel->setMaximumSize(200,100);
	myLabel->setFrameStyle( QFrame::Panel | QFrame::Raised );
	QPixmap *myPixmap = new QPixmap(qembed_findData("dof.png"));

	QLabel *nearDoFLabel = new QLabel("Near: ",tabpage);
	QLabel *farDoFLabel = new QLabel("Far: ",tabpage);
	QLabel *distanceDoFLabel = new QLabel("Distance: ",tabpage);
	QLabel *focalDoFLabel = new QLabel("Focal length: ",tabpage);
	QLabel *apertureDoFLabel = new QLabel("Aperture: ",tabpage);

	nearDoFSpinBox = new DoubleSpinBox(0,10000,3,0.001,tabpage,"Near");
	farDoFSpinBox = new DoubleSpinBox(0,10000,3,0.001,tabpage,"Far");
	distanceDoFSpinBox = new DoubleSpinBox(0,10000,3,0.001,tabpage,"Distance");
	focalDoFSpinBox = new DoubleSpinBox(1,1000,0,1,tabpage,"Focal");
	apertureDoFSpinBox = new DoubleSpinBox(1,32,1,0.1,tabpage,"Aperture");
	
	modeDoFComboBox = new QComboBox(tabpage,"DoFMode");

	QWidget *DoF = new QWidget(tabpage);

	layout->addWidget(myLabel);
	layout->insertSpacing(1,10);
	layout->addLayout(hbox1);
	hbox1->addLayout(vbox0);
	hbox1->addLayout(vbox1);
	hbox1->addLayout(vbox2);
	hbox1->addLayout(vbox3);
	vbox1->addWidget(farDoFLabel);
	vbox1->addWidget(distanceDoFLabel);
	vbox1->addWidget(nearDoFLabel);
	vbox1->addWidget(focalDoFLabel);
	vbox1->addWidget(apertureDoFLabel);
	vbox2->addWidget(farDoFSpinBox);
	vbox2->addWidget(distanceDoFSpinBox);
	vbox2->addWidget(nearDoFSpinBox);
	vbox2->addWidget(focalDoFSpinBox);
	vbox2->addWidget(apertureDoFSpinBox);
	layout->insertSpacing(3,10);
	layout->addLayout(hbox2);
	hbox2->addLayout(vbox21);
	hbox2->addLayout(vbox22);
	hbox2->addLayout(vbox23);
	vbox22->addWidget(modeDoFComboBox);
	layout->addWidget(DoF);

	myLabel->setPixmap(*myPixmap);
	myLabel->setAlignment(Qt::AlignHCenter);

	nearDoFLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	farDoFLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	distanceDoFLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	focalDoFLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	apertureDoFLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

	apertureDoFSpinBox->setPrefix("f/");

	modeDoFComboBox->insertItem("Depth");
	modeDoFComboBox->insertItem("Aperture");

	changedValueMapper->setMapping(farDoFSpinBox,SIGNALS_DOF_FAR);				// Far
	changedValueMapper->setMapping(distanceDoFSpinBox,SIGNALS_DOF_DISTANCE);	// Distance
	changedValueMapper->setMapping(nearDoFSpinBox,SIGNALS_DOF_NEAR);			// Near
	changedValueMapper->setMapping(focalDoFSpinBox,SIGNALS_DOF_FOCALLENGTH);	// Focallength
	changedValueMapper->setMapping(apertureDoFSpinBox,SIGNALS_DOF_APERTURE);	// Aperture
	
	connect(farDoFSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	connect(distanceDoFSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	connect(nearDoFSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	connect(focalDoFSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	connect(apertureDoFSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	
	clickedValueMapper->setMapping(farDoFSpinBox,SIGNALS_DOF_FAR);				// Far
	clickedValueMapper->setMapping(distanceDoFSpinBox,SIGNALS_DOF_DISTANCE);		// Distance
	clickedValueMapper->setMapping(nearDoFSpinBox,SIGNALS_DOF_NEAR);			// Near
	clickedValueMapper->setMapping(focalDoFSpinBox,SIGNALS_DOF_FOCALLENGTH);	// Focallength
	clickedValueMapper->setMapping(apertureDoFSpinBox,SIGNALS_DOF_APERTURE);	// Aperture
	
	connect(farDoFSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));
	connect(distanceDoFSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));
	connect(nearDoFSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));
	connect(focalDoFSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));
	connect(apertureDoFSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));

	connect(modeDoFComboBox,SIGNAL(activated(int)),this,SLOT(changedDoFMode(int)));

	myTabWidget->addTab(tabpage,"DoF");
}

// Field of View
void CameraAssistant::setupTabFoV() {
	QWidget *tabpage = new QWidget(this);
	QVBoxLayout *layout = new QVBoxLayout(tabpage);
	layout->setMargin(5);

	QHBoxLayout *hbox1 = new QHBoxLayout(-1,"hbox1");
	QVBoxLayout *vbox0 = new QVBoxLayout(-1,"vbox0");
	QVBoxLayout *vbox1 = new QVBoxLayout(-1,"vbox1");
	QVBoxLayout *vbox2 = new QVBoxLayout(-1,"vbox2");
	QVBoxLayout *vbox3 = new QVBoxLayout(-1,"vbox3");
	QHBoxLayout *hbox2 = new QHBoxLayout(-1,"hbox2");
	QVBoxLayout *vbox21 = new QVBoxLayout(-1,"vbox21");
	QVBoxLayout *vbox22 = new QVBoxLayout(-1,"vbox22");
	QVBoxLayout *vbox23 = new QVBoxLayout(-1,"vbox23");

	QLabel *myLabel = new QLabel("Label",tabpage);
	myLabel->setMaximumSize(200,100);
	myLabel->setFrameStyle( QFrame::Panel | QFrame::Raised );
	QPixmap *myPixmap = new QPixmap(qembed_findData("fov.png"));

	infoLabel = new QLabel("infoLabel",tabpage);
	infoLabel->setText("<b>( Angle=25 degrees,  )</b>");
	
	QLabel *widthFoVLabel = new QLabel("Width: ",tabpage);
	QLabel *heightFoVLabel = new QLabel("Height: ",tabpage);
	QLabel *distanceFoVLabel = new QLabel("Distance: ",tabpage);
	QLabel *focalFoVLabel = new QLabel("Focal Length: ",tabpage);

	widthFoVSpinBox = new DoubleSpinBox(0,10000,3,0.001,tabpage);
	heightFoVSpinBox = new DoubleSpinBox(0,10000,3,0.001,tabpage);
	distanceFoVSpinBox = new DoubleSpinBox(0,10000,3,0.001,tabpage);
	focalFoVSpinBox = new DoubleSpinBox(0,1000,0,1,tabpage);

	modeFoVComboBox = new QComboBox(tabpage,"FoVMode");

	QWidget *FoV = new QWidget(tabpage);

	layout->addWidget(myLabel);
	layout->addSpacing(10);
	layout->addWidget(infoLabel);
	layout->addSpacing(10);
	layout->addLayout(hbox1);
	hbox1->addLayout(vbox0);
	hbox1->addLayout(vbox1);
	hbox1->addLayout(vbox2);
	hbox1->addLayout(vbox3);
	vbox1->addWidget(widthFoVLabel);
	vbox1->addWidget(heightFoVLabel);
	vbox1->addWidget(distanceFoVLabel);
	vbox1->addWidget(focalFoVLabel);
	vbox2->addWidget(widthFoVSpinBox);
	vbox2->addWidget(heightFoVSpinBox);
	vbox2->addWidget(distanceFoVSpinBox);
	vbox2->addWidget(focalFoVSpinBox);
	layout->addSpacing(10);
	layout->addLayout(hbox2);
	hbox2->addLayout(vbox21);
	hbox2->addLayout(vbox22);
	hbox2->addLayout(vbox23);
	vbox22->addWidget(modeFoVComboBox);
	layout->addWidget(FoV);

	myLabel->setPixmap(*myPixmap);
	myLabel->setAlignment(Qt::AlignHCenter);

	infoLabel->setAlignment(Qt::AlignCenter|Qt::AlignVCenter);
	widthFoVLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	heightFoVLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	distanceFoVLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	focalFoVLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

	modeFoVComboBox->insertItem("Field of View");
	modeFoVComboBox->insertItem("Distance");
	modeFoVComboBox->insertItem("Lens");
	
	changedValueMapper->setMapping(widthFoVSpinBox,SIGNALS_FOV_WIDTH);			// Width
	changedValueMapper->setMapping(heightFoVSpinBox,SIGNALS_FOV_HEIGHT);		// Height
	changedValueMapper->setMapping(distanceFoVSpinBox,SIGNALS_FOV_DISTANCE);	// Distance
	changedValueMapper->setMapping(focalFoVSpinBox,SIGNALS_FOV_FOCALLENGTH);	// Focallength
	connect(widthFoVSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	connect(heightFoVSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	connect(distanceFoVSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	connect(focalFoVSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	
	clickedValueMapper->setMapping(widthFoVSpinBox,SIGNALS_FOV_WIDTH);			// Width
	clickedValueMapper->setMapping(heightFoVSpinBox,SIGNALS_FOV_HEIGHT);			// Height
	clickedValueMapper->setMapping(distanceFoVSpinBox,SIGNALS_FOV_DISTANCE);		// Distance
	clickedValueMapper->setMapping(focalFoVSpinBox,SIGNALS_FOV_FOCALLENGTH);	// Focallength
	connect(widthFoVSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));
	connect(heightFoVSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));
	connect(distanceFoVSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));
	connect(focalFoVSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));
	
	connect(modeFoVComboBox,SIGNAL(activated(int)),this,SLOT(changedFoVMode(int)));

	myTabWidget->addTab(tabpage,"FoV");
}

// Preferences
void CameraAssistant::setupTabPrefs() {
	QWidget *tabpage = new QWidget(this);
	QVBoxLayout *layout = new QVBoxLayout(tabpage);
	layout->setMargin(5);

	QLabel *myLabel = new QLabel("Label",tabpage);
	myLabel->setMaximumSize(200,100);
	myLabel->setFrameStyle( QFrame::Panel | QFrame::Raised );
	QPixmap *myPixmap = new QPixmap(qembed_findData("prefs.png"));

	QPushButton *infoPushButton = new QPushButton(tabpage);
	infoPushButton->setText("?");
//	infoPushButton->setMaximumWidth(20);
	
	QHBoxLayout *hbox1 = new QHBoxLayout(-1,"hbox1");
	QVBoxLayout *vbox0 = new QVBoxLayout(-1,"vbox0");
	QVBoxLayout *vbox1 = new QVBoxLayout(-1,"vbox1");
	QVBoxLayout *vbox2 = new QVBoxLayout(-1,"vbox2");
	QVBoxLayout *vbox3 = new QVBoxLayout(-1,"vbox3");
	QVBoxLayout *vbox4 = new QVBoxLayout(-1,"vbox4");

	QLabel *unitsPrefsLabel = new QLabel("Units:  ",tabpage);
	QLabel *cameracategoryPrefsLabel = new QLabel("Camera category:  ",tabpage);
	QLabel *cameratypePrefsLabel = new QLabel("Camera type:  ",tabpage);
	QLabel *aspectPrefsLabel = new QLabel("Aspect ratio:  ",tabpage);
	QLabel *cocPrefsLabel = new QLabel("Circle of confusion:  ",tabpage);
	QLabel *adjustmentsPrefsLabel = new QLabel("Adjustments:  ",tabpage);

	unitsPrefsComboBox = new QComboBox(tabpage,"Units");
	cameracategoryPrefsComboBox = new QComboBox(tabpage,"Camera Type");
	cameratypePrefsComboBox = new QComboBox(tabpage,"Camera Type");
	aspectPrefsComboBox = new QComboBox(tabpage,"Aspect Ratio");
	cocPrefsSpinBox = new DoubleSpinBox(0,1,5,0.00001,tabpage,"CoC");
	adjustmentsPrefsSpinBox = new DoubleSpinBox(0.001,1,3,0.001,tabpage,"Adjustments");

	QWidget *Preferences = new QWidget(tabpage);

	layout->addWidget(myLabel);
	layout->insertSpacing(1,10);
	layout->addLayout(hbox1);
	hbox1->addLayout(vbox0);
	hbox1->addLayout(vbox1);
	hbox1->addLayout(vbox2);
	hbox1->addLayout(vbox3);
	hbox1->addLayout(vbox4);
	vbox1->addWidget(unitsPrefsLabel);
	vbox1->addWidget(cameracategoryPrefsLabel);
	vbox1->addWidget(cameratypePrefsLabel);
	vbox1->addWidget(aspectPrefsLabel);
	vbox1->addWidget(cocPrefsLabel);
	vbox1->addWidget(adjustmentsPrefsLabel);
	vbox2->addWidget(unitsPrefsComboBox);
	vbox2->addWidget(cameracategoryPrefsComboBox);
	vbox2->addWidget(cameratypePrefsComboBox);
	vbox2->addWidget(aspectPrefsComboBox);
	vbox2->addWidget(cocPrefsSpinBox);
	vbox2->addWidget(adjustmentsPrefsSpinBox);
	vbox3->insertStretch(0);
	vbox3->insertStretch(1);
	vbox3->insertStretch(2);
	vbox3->addWidget(infoPushButton);
	vbox3->insertStretch(4);
	vbox3->insertStretch(5);
	layout->addWidget(Preferences);

	myLabel->setPixmap(*myPixmap);
	myLabel->setAlignment(Qt::AlignHCenter);

	unitsPrefsLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	cameracategoryPrefsLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	cameratypePrefsLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	aspectPrefsLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	cocPrefsLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
	adjustmentsPrefsLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

	unitsPrefsComboBox->insertItem("Metric");
	unitsPrefsComboBox->insertItem("Imperial");
	
	changedValueMapper->setMapping(cocPrefsSpinBox,SIGNALS_PREFS_COC);	// Circle of confusion
	connect(cocPrefsSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	clickedValueMapper->setMapping(cocPrefsSpinBox,SIGNALS_PREFS_COC);	// Circle of confusion
	connect(cocPrefsSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));
	
	connect(unitsPrefsComboBox,SIGNAL(activated(int)),this,SLOT(changedUnits(int)));
	connect(cameracategoryPrefsComboBox,SIGNAL(activated(int)),this,SLOT(changedCameraCategory(int)));
	connect(cameratypePrefsComboBox,SIGNAL(activated(int)),this,SLOT(changedCameraType(int)));
	connect(aspectPrefsComboBox,SIGNAL(activated(int)),this,SLOT(changedAspectRatio(int)));

	changedValueMapper->setMapping(adjustmentsPrefsSpinBox,SIGNALS_PREFS_ADJUSTMENTS);	// Adjustments
	connect(adjustmentsPrefsSpinBox,SIGNAL(valueChanged(double)),changedValueMapper,SLOT(map()));
	clickedValueMapper->setMapping(adjustmentsPrefsSpinBox,SIGNALS_PREFS_ADJUSTMENTS); // Adjustments
	connect(adjustmentsPrefsSpinBox,SIGNAL(clicked()),clickedValueMapper,SLOT(map()));

	connect(infoPushButton,SIGNAL(clicked()),this,SLOT(showInfo()));
	
	myTabWidget->addTab(tabpage,"Prefs");
}

// Filter info.
void CameraAssistant::setupTabFilters() {
	QWidget *tabpage = new QWidget(this);
	QVBoxLayout *layout = new QVBoxLayout(tabpage);
	layout->setMargin(5);

	QLabel *myLabel = new QLabel("Label",tabpage);
	myLabel->setMaximumSize(200,100);
	myLabel->setFrameStyle( QFrame::Panel | QFrame::Raised );
	QPixmap *myPixmap = new QPixmap(qembed_findData("filters.png"));
	myLabel->setPixmap(*myPixmap);
	myLabel->setAlignment(Qt::AlignHCenter);
	
	QListView *filterListView = new QListView(tabpage,"Filtes");
	filterListView->addColumn("Filter");
	filterListView->addColumn("Factor");
	filterListView->addColumn("Info");
	filterListView->setShowSortIndicator(true);
	
	layout->addWidget(myLabel);
	layout->addSpacing(10);
	layout->addWidget(filterListView);
	
	new QListViewItem(filterListView,"85","0.2");
	new QListViewItem(filterListView,"85B","0.2");
	new QListViewItem(filterListView,"85N3","0.5");
	new QListViewItem(filterListView,"LL-D","minor");
	new QListViewItem(filterListView,"81EF","0.1");
	new QListViewItem(filterListView,"N3","0.3");
	new QListViewItem(filterListView,"N6","0.6");
	new QListViewItem(filterListView,"N9","0.9");
	new QListViewItem(filterListView,"N12","1.2");
	new QListViewItem(filterListView,"POLARIZER","0.6");
	new QListViewItem(filterListView,"B-P M 1/2","0.15");
	new QListViewItem(filterListView,"B-P M 1","0.2");
	new QListViewItem(filterListView,"B-P M 3","0.4");
	new QListViewItem(filterListView,"W-P M 1/2","minor");
	new QListViewItem(filterListView,"W-P M 1","minor");
	new QListViewItem(filterListView,"B-NET 1","0.15");
	new QListViewItem(filterListView,"B-NET 3","0.5");
	new QListViewItem(filterListView,"SEPIA 2","<0.6");
	new QListViewItem(filterListView,"812","0.1");
	new QListViewItem(filterListView,"CORAL 1/4","0.15");
	new QListViewItem(filterListView,"CORAL 1/2","0.15");
	new QListViewItem(filterListView,"CORAL 1","0.2");
	new QListViewItem(filterListView,"CORAL 2",">0.2");
	new QListViewItem(filterListView,"CORAL 4","0.3");
	
	filterListView->setColumnWidth(0,58);
	filterListView->setColumnWidth(1,62);
	filterListView->setColumnWidth(2,85);
	
	myTabWidget->addTab(tabpage,"Filters");
}

void CameraAssistant::setupTabAbout() {
	QWidget *tabpage = new QWidget(this);
	QVBoxLayout *layout = new QVBoxLayout(tabpage);
	layout->setMargin(5);
	
	QHBoxLayout *hbox1 = new QHBoxLayout(-1,"hbox1");

	QLabel *myLabel = new QLabel("Label",tabpage);
	myLabel->setMaximumSize(200,100);
	myLabel->setFrameStyle( QFrame::Panel | QFrame::Raised );
	QPixmap *myPixmap = new QPixmap(qembed_findData("about.png"));

	infoAboutTextBrowser = new QTextBrowser(tabpage);
	
	QPushButton *wipePushButton = new QPushButton(tabpage);
	wipePushButton->setText("Recreate configfile.");
//	wipePushButton->setSizePolicy(QSizePolicy(QSizePolicy::Minimum,QSizePolicy::Preferred,FALSE));
	
	QString text;
	text.append("<center>");
	text.append("<b><font size=+1>Camera Assistant</font></b><br>");
	text.append("<b>version: </b>0.1.0<br>");
	text.append("<br>http://cameraassistant.sf.net/<br><br><i>zaurus@bredband.net</i>");
	text.append("</center>");
	infoAboutTextBrowser->setText(text);
	
	layout->addWidget(myLabel);
	layout->insertSpacing(1,10);
	layout->addWidget(infoAboutTextBrowser);
	layout->insertSpacing(3,10);
	layout->addLayout(hbox1);
	hbox1->insertSpacing(0,50);
	hbox1->addWidget(wipePushButton);
	hbox1->insertSpacing(2,50);
	
	myLabel->setPixmap(*myPixmap);
	myLabel->setAlignment(Qt::AlignHCenter);

	connect(wipePushButton,SIGNAL(clicked()),this,SLOT(recreateConfig()));

	myTabWidget->addTab(tabpage,"About");
}

// Clicked on one of the intput values.
void CameraAssistant::clickedValue(int signal) {
	numPad *mypad=0;
	switch(signal) {
		case SIGNALS_DOF_FAR:
			mypad=new numPad(this,"Far focus point",true,far,units);
			if(mypad->exec()) farDoFSpinBox->setValue(tools::convertFrommm(mypad->value,units));
			break;
		case SIGNALS_DOF_DISTANCE:
			mypad=new numPad(this,"Distance",true,distance,units);
			if(mypad->exec()) distanceDoFSpinBox->setValue(tools::convertFrommm(mypad->value,units));
			break;
		case SIGNALS_DOF_NEAR:
			mypad=new numPad(this,"Near focus point",true,near,units);
			if(mypad->exec()) nearDoFSpinBox->setValue(tools::convertFrommm(mypad->value,units));
			break;
		case SIGNALS_DOF_FOCALLENGTH:
			mypad=new numPad(this,"Focal length",true,focallength,UNITS_MM); // Always in mm.
			if(mypad->exec()) focalDoFSpinBox->setValue(mypad->value);
			break;
		case SIGNALS_DOF_APERTURE:
			mypad=new numPad(this,"Aperture",true,aperture,UNITS_MM); // Always in mm.
			if(mypad->exec()) apertureDoFSpinBox->setValue(mypad->value);
			break;
		case SIGNALS_FOV_WIDTH:
			mypad=new numPad(this,"Width",true,imagewidth,units);
			if(mypad->exec()) widthFoVSpinBox->setValue(tools::convertFrommm(mypad->value,units));
			break;
		case SIGNALS_FOV_HEIGHT:
			mypad=new numPad(this,"Height",true,imageheight,units);
			if(mypad->exec()) heightFoVSpinBox->setValue(tools::convertFrommm(mypad->value,units));
			break;
		case SIGNALS_FOV_DISTANCE:
			mypad=new numPad(this,"Distance",true,distance,units);
			if(mypad->exec()) distanceFoVSpinBox->setValue(tools::convertFrommm(mypad->value,units));
			break;
		case SIGNALS_FOV_FOCALLENGTH:
			mypad=new numPad(this,"Focal length",true,focallength,UNITS_MM); // Always in mm.
			if(mypad->exec()) focalFoVSpinBox->setValue(mypad->value);
			break;
		case SIGNALS_PREFS_COC:
			mypad=new numPad(this,"Circle of confusion",true,circle_of_confusion,units+2); // Map meter -> mm, and feet-> inches.
			if(mypad->exec()) cocPrefsSpinBox->setValue(tools::convertFrommm(mypad->value,units+2));
			break;
		case SIGNALS_PREFS_ADJUSTMENTS:
			mypad=new numPad(this,"Adjustments",true,adjustments,units);
			if(mypad->exec()) adjustmentsPrefsSpinBox->setValue(tools::convertFrommm(mypad->value,units));
			break;
	}
	if(mypad) delete mypad;
}

void CameraAssistant::changedValue(int signal) {
	switch(signal) {
		case SIGNALS_DOF_FAR:
			far=tools::convertTomm(farDoFSpinBox->value(),units);
			break;
		case SIGNALS_DOF_DISTANCE:
			distance=tools::convertTomm(distanceDoFSpinBox->value(),units);
			break;
		case SIGNALS_DOF_NEAR:
			near=tools::convertTomm(nearDoFSpinBox->value(),units);
			break;
		case SIGNALS_DOF_FOCALLENGTH:
			focallength=focalDoFSpinBox->value();		// No need to convert to mm, focallength is always in mm.
			break;
		case SIGNALS_DOF_APERTURE:
			aperture=apertureDoFSpinBox->value();	// No need to convert to mm, aperture is always in mm.
			break;
		case SIGNALS_FOV_WIDTH:
			imagewidth=tools::convertTomm(widthFoVSpinBox->value(),units);
			if(aspect>=0) {
				imageheight=(imagewidth/aspect); 
			} else {
				imageheight=(imagewidth*(-aspect));
			}
			break;
		case SIGNALS_FOV_HEIGHT:
			imageheight=tools::convertTomm(heightFoVSpinBox->value(),units);
			if(aspect>=0) {
				imagewidth=imageheight*aspect;
			} else {
				imagewidth=(imageheight/(-aspect));
			}
			break;
		case SIGNALS_FOV_DISTANCE:
			distance=tools::convertTomm(distanceFoVSpinBox->value(),units);
			break;
		case SIGNALS_FOV_FOCALLENGTH:
			focallength=focalFoVSpinBox->value();		// No need to convert to mm, focallength is always in mm.
			break;
		case SIGNALS_PREFS_COC:
			circle_of_confusion=tools::convertTomm(cocPrefsSpinBox->value(),units+2); // mm to mm. Pointless huh? :)
			break;
		case SIGNALS_PREFS_ADJUSTMENTS:
			adjustments=tools::convertTomm(adjustmentsPrefsSpinBox->value(),units);
			farDoFSpinBox->setSteps(tools::convertFrommm(adjustments,units));
			distanceDoFSpinBox->setSteps(tools::convertFrommm(adjustments,units));
			nearDoFSpinBox->setSteps(tools::convertFrommm(adjustments,units));
			widthFoVSpinBox->setSteps(tools::convertFrommm(adjustments,units));
			heightFoVSpinBox->setSteps(tools::convertFrommm(adjustments,units));
			distanceFoVSpinBox->setSteps(tools::convertFrommm(adjustments,units));
			break;
	}
	int page=myTabWidget->currentPageIndex();
	if(page==PAGE_DOF) calculateDoF();
	if(page==PAGE_FOV) calculateFoV();
	updateValues();
}

void CameraAssistant::changedPage() {
	int page=myTabWidget->currentPageIndex();
	if(page==PAGE_DOF) {
		calculateDoF();
	} else  if(page==PAGE_FOV) {
		calculateFoV();
	}
	updateValues();
	myTabWidget->setFocus();  // Workaround for bug, without this, menubutton fires twice in some circumstances.
}

void CameraAssistant::changedDoFMode(int mode) {
	DoFMode=mode;
	switch(mode) {
		case DOFMODE_DEPTH:
			farDoFSpinBox->setEnabled(0);
			distanceDoFSpinBox->setEnabled(1);
			nearDoFSpinBox->setEnabled(0);
			focalDoFSpinBox->setEnabled(1);
			apertureDoFSpinBox->setEnabled(1);
			break;
		case DOFMODE_APERTURE:
			farDoFSpinBox->setEnabled(1);
			distanceDoFSpinBox->setEnabled(0);
			nearDoFSpinBox->setEnabled(1);
			focalDoFSpinBox->setEnabled(1);
			apertureDoFSpinBox->setEnabled(0);
			break;
	}
}

void CameraAssistant::changedFoVMode(int mode) {
	FoVMode=mode;
	switch(mode) {
		case FOVMODE_FOV:
			widthFoVSpinBox->setEnabled(0);
			heightFoVSpinBox->setEnabled(0);
			distanceFoVSpinBox->setEnabled(1);
			focalFoVSpinBox->setEnabled(1);
			break;
		case FOVMODE_DISTANCE:
			widthFoVSpinBox->setEnabled(1);
			heightFoVSpinBox->setEnabled(1);
			distanceFoVSpinBox->setEnabled(0);
			focalFoVSpinBox->setEnabled(1);
			break;
		case FOVMODE_FOCALLENGTH:
			widthFoVSpinBox->setEnabled(1);
			heightFoVSpinBox->setEnabled(1);
			distanceFoVSpinBox->setEnabled(1);
			focalFoVSpinBox->setEnabled(0);
			break;
	}
}

void CameraAssistant::changedUnits(int mode) {
	units=mode;
	updateValues();
}

void CameraAssistant::changedCameraCategory(int value) {
	config->activeCameraCategory=cameracategoryPrefsComboBox->text(value);
	config->configGetCameras();
	
	cameratypePrefsComboBox->clear();
	cameratypePrefsComboBox->insertStringList(config->CameraTypes);
	
	aspectPrefsComboBox->clear();
	aspectPrefsComboBox->insertStringList(config->AspectRatios);
	aspect=config->aspect_ratio;
	filmwidth=config->filmwidth;
	filmheight=config->filmheight;
	
	cocPrefsSpinBox->setValue(config->circle_of_confusion);
	circle_of_confusion=config->circle_of_confusion;
}

void CameraAssistant::changedCameraType(int value) {
	config->activeCameraType=cameratypePrefsComboBox->text(value);
	config->configGetAspectRatios();
	
	aspectPrefsComboBox->clear();
	aspectPrefsComboBox->insertStringList(config->AspectRatios);
	aspect=config->aspect_ratio;
	filmwidth=config->filmwidth;
	filmheight=config->filmheight;
	
	circle_of_confusion=config->circle_of_confusion;
	cocPrefsSpinBox->setValue(circle_of_confusion);
}

void CameraAssistant::changedAspectRatio(int value) {
	config->activeAspectRatio=aspectPrefsComboBox->text(value);
	config->configGetCurrentAspectRatio(); // Parse again to read current aspect ratio value.
	
	aspect=config->aspect_ratio;
	filmwidth=config->filmwidth;
	filmheight=config->filmheight;
}

void CameraAssistant::calculateDoF() {
	switch(DoFMode) {
		case DOFMODE_DEPTH:  // Calculate near,far & depth FROM aperture,distance & focallength.
			hyperfocal = (focallength*focallength)/(aperture*(circle_of_confusion)); // Hyperfocal in mm.
			near = (hyperfocal*(distance)) / (hyperfocal + ((distance)-focallength)); // near in mm.
			far = (hyperfocal*(distance)) / (hyperfocal - ((distance)-focallength)); // far in mm.
			depth = far - near;
			break;
		case DOFMODE_APERTURE:  // Calculate distance & aperture FROM near,far & focallength.
			aperture=(focallength*focallength*(far-near))/(2*circle_of_confusion*far*near);
			hyperfocal = (focallength*focallength)/(aperture*(circle_of_confusion)); // Hyperfocal in mm.
			distance=(2*near*far)/(near+far);
			depth=far-near;
			break;
	}
}

void CameraAssistant::calculateFoV() {
	float magnification;
	if(aspect>0) imageheight=imagewidth/aspect;
	if(aspect<0) imagewidth=imageheight*(-aspect);
	switch(FoVMode) {
		case FOVMODE_FOV:
			magnification=(focallength)/(distance-focallength);
			imagewidth=filmwidth/magnification;
			imageheight=filmheight/magnification;
			break;
		case FOVMODE_DISTANCE:
			magnification=filmwidth/imagewidth;
			distance=(focallength/magnification)+focallength;
			break;
		case FOVMODE_FOCALLENGTH:
			magnification=filmwidth/imagewidth;
			focallength=(magnification*distance)/(1+magnification);
			break;
	}
}

void CameraAssistant::updateValues() {
	int page=myTabWidget->currentPageIndex();
	if (page==PAGE_DOF) {
		if(units==UNITS_M) {
			farDoFSpinBox->setSuffix(" m");
			distanceDoFSpinBox->setSuffix(" m");
			nearDoFSpinBox->setSuffix(" m");
			focalDoFSpinBox->setSuffix(" mm");
			apertureDoFSpinBox->setPrefix("f/");
		} else if(units==UNITS_FEET) {
			farDoFSpinBox->setSuffix(" Ft.");
			distanceDoFSpinBox->setSuffix(" Ft.");
			nearDoFSpinBox->setSuffix(" Ft.");
			focalDoFSpinBox->setSuffix(" mm");
			apertureDoFSpinBox->setPrefix("f/");
		}
		farDoFSpinBox->updateValue(tools::convertFrommm(far,units));
		distanceDoFSpinBox->updateValue(tools::convertFrommm(distance,units));
		nearDoFSpinBox->updateValue(tools::convertFrommm(near,units));
		focalDoFSpinBox->updateValue(focallength);
		apertureDoFSpinBox->updateValue(aperture);
	}
	if(page==PAGE_FOV) {
		if(units==UNITS_M) {
			widthFoVSpinBox->setSuffix(" m");
			heightFoVSpinBox->setSuffix(" m");
			distanceFoVSpinBox->setSuffix(" m");
			focalFoVSpinBox->setSuffix(" mm");
		} else if(units==UNITS_FEET) {
			widthFoVSpinBox->setSuffix(" Ft.");
			heightFoVSpinBox->setSuffix(" Ft.");
			distanceFoVSpinBox->setSuffix(" Ft.");
			focalFoVSpinBox->setSuffix(" mm");
		}
		widthFoVSpinBox->updateValue(tools::convertFrommm(imagewidth,units));
		heightFoVSpinBox->updateValue(tools::convertFrommm(imageheight,units));
		distanceFoVSpinBox->updateValue(tools::convertFrommm(distance,units));
		focalFoVSpinBox->updateValue(focallength);
		
		float w_angle=2*atan(filmwidth/(2*focallength))*180/M_PI;
		float h_angle=2*atan(filmheight/(2*focallength))*180/M_PI;
		QString angle;
		angle.sprintf("<b>( Angle of view   -   H: %.1f   /  V: %.1f  )</b>",w_angle,h_angle);
		infoLabel->setText(angle);
	}
	if(page==PAGE_PREFS) {
		if(units==UNITS_M) {
			cocPrefsSpinBox->setSuffix(" mm");
			adjustmentsPrefsSpinBox->setSuffix(" m");
		} else if(units==UNITS_FEET) {
			cocPrefsSpinBox->setSuffix(" In.");
			adjustmentsPrefsSpinBox->setSuffix(" Ft.");
		}
		cocPrefsSpinBox->updateValue(tools::convertFrommm(circle_of_confusion,units+2));
		adjustmentsPrefsSpinBox->updateValue(tools::convertFrommm(adjustments,units));
	}
}

void CameraAssistant::showInfo() {
	double width=0,height=0,temp=0;
	if(aspect>0) {
		width=filmwidth;
		height=width/aspect;
		temp=aspect;
	}
	if(aspect<0) {
		height=filmheight;
		width=filmheight/(-aspect);
		temp=-aspect;
	}
	QString message;
	message.append("Aperture size: \n");
	message.append(QString::number(width)+" mm * "+QString::number(height)+" mm\n");
	message.append("("+QString::number(tools::convertFrommm(width,UNITS_INCHES))+" inches * "+QString::number(tools::convertFrommm(height,UNITS_INCHES))+" inches)\n");
	message.append("\nAspect ratio: \n");
	message.append(QString::number(temp)+":1\n");
	QMessageBox mb("Camera Info",message,QMessageBox::Information,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
	mb.exec();
}

void CameraAssistant::recreateConfig() {
	QMessageBox mb("Recreate configfile","Are you REALLY sure you want to \nrebuild the configfile? \n\nAll changes will be lost.",QMessageBox::Warning,QMessageBox::Yes,QMessageBox::No,QMessageBox::NoButton);
	if(mb.exec()==QMessageBox::Yes) {
		qWarning("Ok..");
		config->recreate();
		circle_of_confusion=config->circle_of_confusion;
		filmwidth=config->filmwidth;
		filmheight=config->filmheight;
		aspect=config->aspect_ratio;
		adjustments=config->adjustments;
	}
}

bool CameraAssistant::eventFilter(QObject *object, QEvent *event) {
	if(event->type()==QEvent::KeyPress) {
		QKeyEvent *k=(QKeyEvent *)event;
		//qWarning("Key pressed: %d",k->key());
		if(k->key()==Qt::Key_F11) {
			units=(units+1)%2;	// Switch units
			unitsPrefsComboBox->setCurrentItem(units);
			changedUnits(units);
			return false;
		}
	} 
/*
	else if(event->type()==QEvent::MouseButtonPress) {
		if(((QMouseEvent*)event)->button()==LeftButton) {
			QWidget *focus=this->focusWidget();
			if(focus==distanceDoFSpinBox) qWarning("Right Mouse Button");
		}
	}
*/
	return QMainWindow::eventFilter(object,event);
}

