#ifndef CA_H
#define CA_H

#include <qobject.h>
#include <qmainwindow.h>
#include <qtabwidget.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlist.h>
#include <qsignalmapper.h>
#include <qtabwidget.h>
#include <qtextbrowser.h>

#include "doublespinbox.h"
#include "configparser.h"
#include "numpad.h"
#include "tools.h"

#include <qpe/qpeapplication.h>

class CameraAssistant : public QMainWindow
{
	Q_OBJECT

public:
	CameraAssistant( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
	~CameraAssistant();

private:
	enum PAGE {
		PAGE_DOF,
		PAGE_FOV,
		PAGE_PREFS,
		PAGE_FILTERS,
		PAGE_ABOUT
	};
	enum UNITS {
		UNITS_M,
		UNITS_FEET,
		UNITS_MM,
		UNITS_INCHES
	};
	enum DOFMODE {
		DOFMODE_DEPTH,
		DOFMODE_APERTURE
	};
	enum FOVMODE {
		FOVMODE_FOV,
		FOVMODE_DISTANCE,
		FOVMODE_FOCALLENGTH
	};
	enum SIGNALS {
		SIGNALS_DOF_FAR,
		SIGNALS_DOF_DISTANCE,
		SIGNALS_DOF_NEAR,
		SIGNALS_DOF_FOCALLENGTH,
		SIGNALS_DOF_APERTURE,
		SIGNALS_FOV_WIDTH,
		SIGNALS_FOV_HEIGHT,
		SIGNALS_FOV_DISTANCE,
		SIGNALS_FOV_FOCALLENGTH,
		SIGNALS_PREFS_COC,
		SIGNALS_PREFS_ADJUSTMENTS
	};
	double far,distance,near,depth,hyperfocal,focallength,aperture;
	double filmwidth,filmheight;
	double imagewidth,imageheight;
	int units;
	double aspect,circle_of_confusion,adjustments;
	int DoFMode;
	int FoVMode;

	ConfigParser *config;

	QSignalMapper *clickedValueMapper;
	QSignalMapper *changedValueMapper;
	
	QTabWidget *myTabWidget;

	DoubleSpinBox *nearDoFSpinBox;
	DoubleSpinBox *farDoFSpinBox;
	DoubleSpinBox *distanceDoFSpinBox;
	DoubleSpinBox *focalDoFSpinBox;
	DoubleSpinBox *apertureDoFSpinBox;
	QComboBox *modeDoFComboBox;

	QLabel *infoLabel;
	DoubleSpinBox *widthFoVSpinBox;
	DoubleSpinBox *heightFoVSpinBox;
	DoubleSpinBox *distanceFoVSpinBox;
	DoubleSpinBox *focalFoVSpinBox;
	QComboBox *modeFoVComboBox;

	QComboBox *unitsPrefsComboBox;
	QComboBox *cameracategoryPrefsComboBox;
	QComboBox *cameratypePrefsComboBox;
	QComboBox *aspectPrefsComboBox;
	DoubleSpinBox *cocPrefsSpinBox;
	DoubleSpinBox *adjustmentsPrefsSpinBox;
	
	QTextBrowser *infoAboutTextBrowser;

	void setupTabDoF();
	void setupTabFoV();
	void setupTabFilters();
	void setupTabPrefs();
	void setupTabAbout();
	void calculateDoF();
	void calculateFoV();
	void updateValues();
	
private slots:
	void changedDoFMode(int mode = 0);
	void changedFoVMode(int mode = 0);
	void changedUnits(int mode = 0);
	void changedCameraCategory(int value);
	void changedCameraType(int value);
	void changedAspectRatio(int value);
	
	void changedPage(void);
	void clickedValue(int signal);
	void changedValue(int signal);
	void showInfo(void);
	void recreateConfig(void);
	bool eventFilter(QObject *object, QEvent *event);
};

#endif // CA_H
