#include "configparser.h"
#include "defaultconfig.h"

bool ConfigParser::startDocument() {
	countCameraCategory=0;
	countCameraType=0;
	countAspectRatio=0;
	if(parseMode>=1) {
		AspectRatios.clear();
		activeAspectRatio.truncate(0);
		if(parseMode>=2) {
			CameraTypes.clear();
			activeCameraType.truncate(0);
			if(parseMode>=3) {
				Categories.clear();
				activeCameraCategory.truncate(0);
			}
		}
	}
	return TRUE;
}

bool ConfigParser::endDocument() {
	init=0;
	return TRUE;
}

bool ConfigParser::startElement( const QString&, const QString&, const QString& qName, const QXmlAttributes& attr) {
	if(qName=="Settings") settings=1;
	if(qName=="CameraCategory") {
		parsingCameraCategory=attr.value("name");
		if(activeCameraCategory.isEmpty()) activeCameraCategory=attr.value("name");
		if(parseMode>=3) Categories.append(attr.value("name"));
		if(activeCameraCategory==parsingCameraCategory) {
			indexCameraCategory=countCameraCategory;
		}
		countCameraCategory++;
	}
	if(qName=="CameraType") {
		parsingCameraType=attr.value("name");
		if(parseMode>=2) {
			if(parsingCameraCategory==activeCameraCategory) {
				if(activeCameraType.isEmpty()) activeCameraType=attr.value("name");
				CameraTypes.append(attr.value("name"));
			}
		}
		if(activeCameraType==parsingCameraType && activeCameraCategory==parsingCameraCategory) {
			indexCameraType=countCameraType;
			circle_of_confusion=attr.value("coc").toDouble();
			filmwidth=attr.value("width").toDouble();
			filmheight=attr.value("height").toDouble();
		}
		countCameraType++;
	}
	if(qName=="AspectRatio") {
		parsingAspectRatio=attr.value("name");
		if(parseMode>=1) {
			if(parsingCameraType==activeCameraType && parsingCameraCategory==activeCameraCategory) {
				if(activeAspectRatio.isEmpty()) activeAspectRatio=attr.value("name");
				AspectRatios.append(attr.value("name"));
			}
		}
		if(activeAspectRatio==parsingAspectRatio && activeCameraType==parsingCameraType && activeCameraCategory==parsingCameraCategory) {
			indexAspectRatio=countAspectRatio;
			if(!(attr.value("width").isEmpty())) filmwidth=attr.value("width").toDouble();
			if(!(attr.value("height").isEmpty())) filmheight=attr.value("height").toDouble();
			if(attr.value("aspectratio").isEmpty()) {
				aspect_ratio=filmwidth/filmheight;
			} else {
				aspect_ratio=attr.value("aspectratio").toFloat();
			}
		}
		countAspectRatio++;
	}

	if(settings && init) {
		if(qName=="defaultCameraCategory") activeCameraCategory=attr.value("value");
		if(qName=="defaultCameraType") activeCameraType=attr.value("value");
		if(qName=="defaultAspectRatio") activeAspectRatio=attr.value("value");
		if(qName=="defaultAdjustments") adjustments=attr.value("value").toDouble();
	}

	return TRUE;
}

bool ConfigParser::endElement( const QString&, const QString&, const QString& qName) {
	if(qName=="Settings")  settings=0;
	if(qName=="CameraCategory") {
		countCameraType=0;
		parsingCameraCategory.truncate(0);
	}
	if(qName=="CameraType") {
		if(countAspectRatio==0 && parsingCameraType==activeCameraType) {
			// If no aspect ratio is defined, create one called "Full aperture.", which uses the full image area.
			AspectRatios.append("Full aperture");
			aspect_ratio=filmwidth/filmheight;
		}
		countAspectRatio=0;
		parsingCameraType.truncate(0);
	}
	if(qName=="AspectRatio") {
		parsingAspectRatio.truncate(0);
	}
	return TRUE;
}

void ConfigParser::recreate() {
	QFile xmlFile(Global::applicationFileName("ca","ca.xml"));
	xmlFile.remove();
	init=1; settings=0;
	configOpen();
}

bool ConfigParser::configOpen() {
	QFile xmlFile(Global::applicationFileName("ca","ca.xml"));
	if(!xmlFile.exists()) {
		xmlFile.open(IO_WriteOnly);
		QTextStream ts(&xmlFile);

		ts << defaultConfig;
		xmlFile.close();
	}

	adjustments=1;
	parseMode=3; // setup all three widgets.
	configSource=new QXmlInputSource( xmlFile );
	configReader=new QXmlSimpleReader();
	configReader->setContentHandler( this );
	configReader->parse( *configSource );
	return TRUE;
}

bool ConfigParser::configClose() {
	return TRUE;
}

void ConfigParser::configGetCategories() {
	parseMode=3;
	configReader->parse(*configSource);
}

void ConfigParser::configGetCameras() {
	parseMode=2;
	configReader->parse(*configSource);
}

void ConfigParser::configGetAspectRatios() {
	parseMode=1;
	configReader->parse(*configSource);
}

void ConfigParser::configGetCurrentAspectRatio() {
	parseMode=0;
	configReader->parse(*configSource);
}
