#include "numpad.h"

numPad::numPad(QWidget *parent, const char *name, bool modal, double original_value, int startUnits,WFlags f) : QDialog(parent, name, modal, f) {
	value=original_value;
	originalValue=original_value;
	relative=0;
	units=startUnits;
	setCaption(name);
	resize(200,200);
	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->setMargin(5);
	
	QHBoxLayout *hbox1 = new QHBoxLayout(-1,"hbox1");
	QHBoxLayout *hbox2 = new QHBoxLayout(-1,"hbox2");
	QHBoxLayout *hbox3 = new QHBoxLayout(-1,"hbox3");
	QHBoxLayout *hbox4 = new QHBoxLayout(-1,"hbox4");
	QHBoxLayout *hbox5 = new QHBoxLayout(-1,"hbox5");

	myComboBox = new QComboBox("Units",this);
	myComboBox->insertItem("Meters");
	myComboBox->insertItem("Feet");
	myComboBox->insertItem("mm");
	myComboBox->insertItem("Inches");
	myComboBox->setCurrentItem(units);
	myLineEdit = new QLineEdit("Value",this);
	myLineEdit->setValidator(new QDoubleValidator(this));
	QString myvalue;
	myvalue.setNum(tools::convertFrommm(value,units));
	myLineEdit->setText(myvalue);
	myLineEdit->setAlignment(Qt::AlignRight);

	QPushButton *myPushButton1 = new QPushButton(" 1 ",this);
	QPushButton *myPushButton2 = new QPushButton(" 2 ",this);
	QPushButton *myPushButton3 = new QPushButton(" 3 ",this);
	QPushButton *myPushButton4 = new QPushButton(" 4 ",this);
	QPushButton *myPushButton5 = new QPushButton(" 5 ",this);
	QPushButton *myPushButton6 = new QPushButton(" 6 ",this);
	QPushButton *myPushButton7 = new QPushButton(" 7 ",this);
	QPushButton *myPushButton8 = new QPushButton(" 8 ",this);
	QPushButton *myPushButton9 = new QPushButton(" 9 ",this);

	QPushButton *myPushButtonPoint = new QPushButton(" . ",this);
	QPushButton *myPushButton0 = new QPushButton(" 0 ",this);
	QPushButton *myPushButtonHelp = new QPushButton(" ? ",this);
	myPushButtonHelp->setDisabled(true);

	myPushButtonPlus = new QPushButton(" + ",this);
	myPushButtonPlus->setToggleButton(true);
	myPushButtonMinus = new QPushButton(" - ",this);
	myPushButtonMinus->setToggleButton(true);
	QPushButton *myPushButtonClear = new QPushButton("CLR",this);
	QPushButton *myPushButtonRevert = new QPushButton("RVT",this);
	layout->addLayout(hbox1);
	layout->addLayout(hbox2);
	layout->addLayout(hbox3);
	layout->addLayout(hbox4);
	layout->addLayout(hbox5);
	hbox1->addWidget(myLineEdit);
	hbox1->insertSpacing(1,10);
	hbox1->addWidget(myComboBox);
	hbox2->addWidget(myPushButtonPlus);
	hbox2->addWidget(myPushButton7);
	hbox2->addWidget(myPushButton8);
	hbox2->addWidget(myPushButton9);
	hbox3->addWidget(myPushButtonMinus);
	hbox3->addWidget(myPushButton4);
	hbox3->addWidget(myPushButton5);
	hbox3->addWidget(myPushButton6);
	hbox4->addWidget(myPushButtonClear);
	hbox4->addWidget(myPushButton1);
	hbox4->addWidget(myPushButton2);
	hbox4->addWidget(myPushButton3);
	hbox5->addWidget(myPushButtonRevert);
	hbox5->addWidget(myPushButtonPoint);
	hbox5->addWidget(myPushButton0);
	hbox5->addWidget(myPushButtonHelp);

	signalMapper=new QSignalMapper(this);
	connect(signalMapper,SIGNAL(mapped(int)),this,SLOT(clickedButtons(int)));
	mapButton(myPushButton0,0);
	mapButton(myPushButton1,1);
	mapButton(myPushButton2,2);
	mapButton(myPushButton3,3);
	mapButton(myPushButton4,4);
	mapButton(myPushButton5,5);
	mapButton(myPushButton6,6);
	mapButton(myPushButton7,7);
	mapButton(myPushButton8,8);
	mapButton(myPushButton9,9);
	mapButton(myPushButtonPoint,10);
	mapButton(myPushButtonClear,11);
	mapButton(myPushButtonRevert,12);
	mapButton(myPushButtonPlus,13);
	mapButton(myPushButtonMinus,14);
	
	connect(myLineEdit,SIGNAL(textChanged(const QString &)),this,SLOT(changeValue(const QString &)));
	connect(myComboBox,SIGNAL(activated(int)),this,SLOT(changeUnits(int)));
	
	myLineEdit->setFocus();
	myLineEdit->setSelection(0,100);
}

void numPad::mapButton(QPushButton *button,int signal) {
	signalMapper->setMapping(button,signal);
	connect(button,SIGNAL(clicked()),signalMapper,SLOT(map()));
}

void numPad::changeValue(const QString &text) {
	value=tools::convertTomm(text.toDouble(),units);
}

void numPad::changeUnits(int newUnits) {
	double value=myLineEdit->text().toDouble();
	value=tools::convertTomm(value,units);
	units=newUnits;
	value=tools::convertFrommm(value,units);
	QString myvalue;
	myvalue.setNum(value);
	myLineEdit->setText(myvalue);
}

void numPad::clickedButtons(int signal) {
	QString mystring;
	switch(signal) {
		case 10: // Point
			myLineEdit->insert(".");
			break;
		case 11: // Clear
			myLineEdit->setFocus();
			myLineEdit->setText("");
			break;
		case 12: // Revert
			mystring.setNum(tools::convertFrommm(originalValue,units));
			myLineEdit->setText(mystring);
		case 13:	// Plus
			if(myPushButtonPlus->isOn())  myPushButtonMinus->setOn(false);
			break;
		case 14:	// Minus
			if(myPushButtonMinus->isOn())  myPushButtonPlus->setOn(false);
			break;
		default:
			mystring.setNum(signal);
			myLineEdit->insert(mystring);
			break;
	}
}

void numPad::accept() {
	if(myPushButtonPlus->isOn()) value=originalValue+value;
	if(myPushButtonMinus->isOn()) value=originalValue-value;
	QDialog::accept();
}
